/*!
 * kinetik - job model
 * Copyright(c) 2012 Jake Luer <jake@qualiancy.com>
 * MIT Licensed
 */

/*!
 * External module dependancies
 */

var Seed = require('seed')
  , debug = require('debug')('kinetik:job');

/*!
 * Extend Seed.Model
 */

module.exports = Seed.Model.extend('job', {

    /*!
     * # process (task, callback)
     *
     * Process a given job using the incoming passed
     * Task. Mark as succeeded or failed.
     *
     * @param {Seed.Model} task
     * @param {Function} callback upon completion
     * @api private
     */

    process: function (task, cb) {
      var self = this
        , action = task.get('action')
        , job = {
              id: this.id
            , data: this.get('data')
          };

      debug('processing %s', this.get('task'));
      this.set('status', 'processing');
      this.save(function (err) {
        if (err) return cb(err);
        action(job, function (fail) {
          if (fail) {
            if (fail.message) self.set('error', fail.message);
            self.set('status', 'failed');
            task.emit('error', self, fail);
          } else {
            self.set('status', 'completed');
            task.emit('completed', self);
          }
          debug('job %s finished with status: %s'
            , self.get('task')
            , self.get('status')
          );
          self.save(cb);
        });
      });
    }

    /**
     * # cancel (task, callback)
     *
     * Cancel the job and emit `cancel` event.
     *
     * @param {Seed.Model} task
     * @param {Function} callback upon completion
     * @api private
     */

  , cancel: function (task, cb) {
      var self = this;
      this.set('status', 'cancelled');
      if (test) task.emit('cancelled', this);
      this.save(cb);
    }
});
